@@ -6,6 +6,7 @@
  * The MIT License (MIT)
  *
  * Copyright (c) 2019-2020,2022 Omics Data Automation, Inc.
+ * Copyright (c) 2023 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -48,9 +49,6 @@
 #include <vector>
 #include <functional>
 
-[[deprecated("Use genomicsdb::version instead")]]
-GENOMICSDB_EXPORT std::string genomicsdb_version();
-
 typedef std::pair<uint64_t, uint64_t> interval_t;
 
 typedef struct genomic_interval_t {
@@ -144,7 +142,7 @@ typedef struct genomic_field_t {
     return *(reinterpret_cast<char *>(const_cast<void *>(ptr)) + offset);
   }
   inline std::string str_value() const {
-    return std::string(reinterpret_cast<const char *>(ptr)).substr(0, num_elements);
+    return std::string(reinterpret_cast<const char *>(ptr), num_elements);
   }
   inline std::string cpp_str_value_at(uint64_t offset) const {
     check_offset(offset);
@@ -242,6 +240,9 @@ class Variant;
 class VariantCall;
 class VariantQueryConfig;
 
+// Utility funtion to resolve GT w.r.t REF/ALT
+GENOMICSDB_EXPORT const std::string resolve_gt(const std::vector<genomic_field_t>& genomic_fields);
+
 class GENOMICSDB_EXPORT GenomicsDBVariantCallProcessor {
  public:
   GenomicsDBVariantCallProcessor() {};
@@ -267,6 +268,34 @@ class GENOMICSDB_EXPORT GenomicsDBVariantCallProcessor {
   std::shared_ptr<std::map<std::string, genomic_field_type_t>> m_genomic_field_types;
 };
 
+class GENOMICSDB_EXPORT JSONVariantCallProcessor : public GenomicsDBVariantCallProcessor {
+ public:
+  enum payload_t {all=0, all_by_calls=1, samples_with_ncalls=2, just_ncalls=3, just_samples=4};
+  JSONVariantCallProcessor() : JSONVariantCallProcessor(all) {}
+  JSONVariantCallProcessor(payload_t payload_mode);
+  ~JSONVariantCallProcessor();
+  void set_payload_mode(payload_t payload_mode) { m_payload_mode = payload_mode; }
+  void process(const interval_t& interval);
+  void process(const std::string& sample_name,
+               const int64_t* coordinates,
+               const genomic_interval_t& genomic_interval,
+               const std::vector<genomic_field_t>& genomic_fields);
+  std::string construct_json_output();
+ private:
+  payload_t m_payload_mode;
+  bool m_is_initialized = false;
+  void *m_json_document;
+  // payload num_calls
+  int64_t m_num_calls = 0ul;
+  // payload samples set
+  std::unique_ptr<std::set<std::string>> m_samples_set;
+  // payload samples_with_ncalls
+  std::unique_ptr<std::map<std::string, int64_t>> m_samples;
+  // payload all
+  std::vector<std::string> m_field_names;
+  std::unique_ptr<std::map<std::string, std::vector<void *>>> m_samples_info;
+};
+
 class GENOMICSDB_EXPORT GenomicsDBVariantProcessor {
  public:
   GenomicsDBVariantProcessor() {};
@@ -298,7 +327,7 @@ class GENOMICSDB_EXPORT GenomicsDBVariantProcessor {
  */
 class GenomicsDB {
  public:
-  enum GENOMICSDB_EXPORT query_config_type_t { JSON_FILE=0, JSON_STRING=1, PROTOBUF_BINARY_STRING=2 };
+  enum GENOMICSDB_EXPORT query_config_type_t { NONE=-1, JSON_FILE=0, JSON_STRING=1, PROTOBUF_BINARY_STRING=2 };
 
   /**
    * Constructor to the GenomicsDB Query API
@@ -317,8 +346,8 @@ class GenomicsDB {
 
   /**
    * Constructor to the GenomicsDB Query API with configuration json files
-   *   query_configuration - describe the query configuration in either a JSON file or string
-   *   query_configuration_type - type of query configuration, could be a JSON_FILE or JSON_STRING
+   *   query_configuration - describe the query configuration in either a JSON file or JSON string or protobuf binary
+   *   query_configuration_type - type of query configuration, could be a JSON_FILE or JSON_STRING or PROTOBUF_BINARY_STRING
    *   loader_config_json_file, optional - describe the loader configuration in a JSON file.
    *           If a configuration key exists in both the query and the loader configuration, the query
    *           configuration takes precedence
@@ -383,10 +412,19 @@ class GenomicsDB {
   GENOMICSDB_EXPORT GenomicsDBVariantCalls query_variant_calls();
 
   /**
-   * Query using set configuration for variant calls. Useful when using parallelism paradigms(MPI, Intel TBB)
+   * Query with a configuration describing the subset for variant calls. Useful with paradigms like MPI, Intel TBB
+   * and when a GenomicsDB instance is cached with multiple, concurrent query_variant_calls with different
+   * subset configurations.
    * Variant Calls are similar to GACall in GA4GH API.
+   *   processor custom processor to process variant calls
+   *   query_configuration protobuf export configuration as binary string, optional. If not specified, the
+   *                       configuration specified during class construction will be used with the query 
+   *   query_configuration_type Type of configuration, Currently only PROTOBUF_BINARY_STRING is supported and
+   *                            an exception is throwm for other types.
    */
-  GENOMICSDB_EXPORT GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor& processor);
+  GENOMICSDB_EXPORT GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor& processor,
+                                                               const std::string& query_configuration,
+                                                               const query_config_type_t query_configuration_type);
 
   /**
    * Generate multi-sample vcf files from GenomicsDB in the Broad GVCF format for given array constrained by
