@@ -6,6 +6,7 @@
  * The MIT License (MIT)
  *
  * Copyright (c) 2019-2020,2022 Omics Data Automation, Inc.
+ * Copyright (c) 2023 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -64,10 +65,6 @@
 #define TO_GENOMICSDB_VARIANT_CALL_VECTOR(X) (reinterpret_cast<std::vector<genomicsdb_variant_call_t> *>(static_cast<void *>(X)))
 
 
-std::string genomicsdb_version() {
-  return genomicsdb::version();
-}
-
 #define VERIFY(X) if(!(X)) throw GenomicsDBException(#X);
 
 void check(const std::string& workspace,
@@ -77,8 +74,8 @@ void check(const std::string& workspace,
 }
 
 void check(const std::string& workspace,
-                  const uint64_t segment_size,
-                  const std::string& callset_mapping_file,
+           const uint64_t segment_size,
+           const std::string& callset_mapping_file,
            const std::string& vid_mapping_file) {
   check(workspace, segment_size);
   VERIFY(!callset_mapping_file.empty() && "Callset Mapping File cannot be empty");
@@ -135,21 +132,21 @@ GenomicsDB::GenomicsDB(const std::string& query_configuration,
       query_config->read_from_JSON_string(query_configuration, concurrency_rank); break;
     case PROTOBUF_BINARY_STRING: {
       query_config->read_from_PB_binary_string(query_configuration, concurrency_rank);
-
-      // Determine which chromosomes are included in the query and pass the list to the annotation service 
-      // so it knows not to bother opening chromosome-specific vcfs that aren't relevant. 
-      // ColumnRange is std::pair<int64_t, int64_t> see genomicsdb.h:71
-      ColumnRange column_partition = query_config->get_column_partition(concurrency_rank);
-      // ContigIntervalTuple is std::tuple<std::string, int64_t, int64_t> see vid_mapper.cc:33
-      std::vector<ContigIntervalTuple> contig_intervals = query_config->get_vid_mapper().get_contig_intervals_for_column_partition(column_partition.first, column_partition.second, true);
-      std::set<std::string> contigs;
-      for (auto interval : contig_intervals) {
-        contigs.insert(std::get<0>(interval));
+      if (query_config->has_annotation_sources()) {
+        // Determine which chromosomes are included in the query and pass the list to the annotation service
+        // so it knows not to bother opening chromosome-specific vcfs that aren't relevant.
+        // ColumnRange is std::pair<int64_t, int64_t> see genomicsdb.h:71
+        ColumnRange column_partition = query_config->get_column_partition(concurrency_rank);
+        // ContigIntervalTuple is std::tuple<std::string, int64_t, int64_t> see vid_mapper.cc:33
+        std::vector<ContigIntervalTuple> contig_intervals = query_config->get_vid_mapper().get_contig_intervals_for_column_partition(column_partition.first, column_partition.second, true);
+        std::set<std::string> contigs;
+        for (auto interval : contig_intervals) {
+          contigs.insert(std::get<0>(interval));
+        }
+        // Create an annotationService class
+        m_annotation_service = new AnnotationService(query_configuration, contigs);
+        AnnotationService* annotation_service = TO_ANNOTATION_SERVICE(m_annotation_service);
       }
-
-      // Create an annotationService class.
-      m_annotation_service = new AnnotationService(query_configuration, contigs);
-      AnnotationService* annotation_service = TO_ANNOTATION_SERVICE(m_annotation_service);
       break;
     }
     default:
@@ -160,7 +157,9 @@ GenomicsDB::GenomicsDB(const std::string& query_configuration,
         query_config->get_segment_size());
 
   // Discard intervals not part of this partition
-  query_config->subset_query_column_ranges_based_on_partition(loader_config, concurrency_rank);
+  if (!loader_configuration_json_file.empty()) {
+    query_config->subset_query_column_ranges_based_on_partition(loader_config, concurrency_rank);
+  }
 
   // Create storage manager
   m_storage_manager = new VariantStorageManager(query_config->get_workspace(concurrency_rank), query_config->get_segment_size(), query_config->enable_shared_posixfs_optimizations());
@@ -232,14 +231,16 @@ GenomicsDBVariants GenomicsDB::query_variants(const std::string& array,
   VariantQueryConfig *base_query_config = TO_VARIANT_QUERY_CONFIG(m_query_config);
   VariantQueryConfig query_config(*base_query_config);
   query_config.set_array_name(array);
-  if (column_ranges.size() > 0) {
+  if (column_ranges.size() == 0) {
+    query_config.set_query_column_ranges(SCAN_FULL);
+  } else {
     query_config.set_query_column_ranges(column_ranges);
   }
   if (row_ranges.size() > 0) {
     query_config.set_query_row_ranges(row_ranges);
   }
 
-  query_config.validate();
+  query_config.validate(m_concurrency_rank);
 
   return GenomicsDBVariants(TO_GENOMICSDB_VARIANT_VECTOR(query_variants(array, &query_config)),
                             create_genomic_field_types(query_config, m_annotation_service, true));
@@ -343,10 +344,16 @@ GenomicsDBVariantCalls GenomicsDB::query_variant_calls(GenomicsDBVariantCallProc
   VariantQueryConfig *base_query_config = TO_VARIANT_QUERY_CONFIG(m_query_config);
   VariantQueryConfig query_config(*base_query_config);
   query_config.set_array_name(array);
-  query_config.set_query_column_ranges(column_ranges);
-  query_config.set_query_row_ranges(row_ranges);
+  if (column_ranges.size() == 0) {
+    query_config.set_query_column_ranges(SCAN_FULL);
+  } else {
+    query_config.set_query_column_ranges(column_ranges);
+  }
+  if (row_ranges.size() > 0) {
+    query_config.set_query_row_ranges(row_ranges);
+  }
 
-  query_config.validate();
+  query_config.validate(m_concurrency_rank);
 
   return GenomicsDBVariantCalls(TO_GENOMICSDB_VARIANT_CALL_VECTOR(query_variant_calls(array, &query_config, processor)),
                                 create_genomic_field_types(query_config, m_annotation_service));
@@ -354,14 +361,49 @@ GenomicsDBVariantCalls GenomicsDB::query_variant_calls(GenomicsDBVariantCallProc
 
 GenomicsDBVariantCalls GenomicsDB::query_variant_calls() {
   GenomicsDBVariantCallProcessor processor;
-  return query_variant_calls(processor);
+  return query_variant_calls(processor, "", NONE);
 }
 
-GenomicsDBVariantCalls GenomicsDB::query_variant_calls(GenomicsDBVariantCallProcessor& processor) {
-  VariantQueryConfig* query_config = TO_VARIANT_QUERY_CONFIG(m_query_config);
-  const std::string& array = query_config->get_array_name(m_concurrency_rank);
-  return GenomicsDBVariantCalls(TO_GENOMICSDB_VARIANT_CALL_VECTOR(query_variant_calls(array, query_config, processor)),
-                                create_genomic_field_types(*query_config, m_annotation_service));
+GenomicsDBVariantCalls GenomicsDB::query_variant_calls(GenomicsDBVariantCallProcessor& processor,
+                                                       const std::string& query_configuration,
+                                                       const query_config_type_t query_configuration_type) {
+  try {
+    // Create Variant Config for given concurrency_rank
+    VariantQueryConfig *base_query_config = TO_VARIANT_QUERY_CONFIG(m_query_config);
+    VariantQueryConfig query_config(*base_query_config);
+
+    if (query_configuration.size() > 0) {
+      if (query_configuration_type != PROTOBUF_BINARY_STRING) {
+        logger.fatal(GenomicsDBException(), "Unsupported query configuration type={} specified to query_variant_calls()",
+                     query_configuration_type);
+      }
+
+      genomicsdb_pb::QueryConfiguration query_config_pb;
+      auto status = query_config_pb.ParseFromString(query_configuration);
+      if(!status || !query_config_pb.IsInitialized()) {
+        logger.fatal(GenomicsDBException("Could not parse query_configuration. Only protobuf QueryConfiguration binary strings accepted as input argument"));
+      }
+      query_config.subset_query_from_PB(&query_config_pb, m_concurrency_rank);
+    }
+
+    if (!query_config.has_array_name(m_concurrency_rank)) {
+      auto array_names = TileDBUtils::get_array_names(query_config.get_workspace(m_concurrency_rank));
+      if (array_names.size() == 1) {
+        query_config.set_array_name(array_names[0]);
+      } else {
+        // TODO: Should figure out the array names based on contig intervals as done on the JAVA side
+        logger.fatal(GenomicsDBConfigException("Query configuration must either have array_name set or should be a single array in the workspace for now"));
+      }
+    }
+
+    query_config.validate(m_concurrency_rank);
+    
+    return GenomicsDBVariantCalls(TO_GENOMICSDB_VARIANT_CALL_VECTOR(
+        query_variant_calls(query_config.get_array_name(m_concurrency_rank), &query_config, processor)),
+                                  create_genomic_field_types(query_config, m_annotation_service));
+  } catch(const GenomicsDBIteratorException& e) {
+    throw GenomicsDBException(e.what());
+  }
 }
 
 VidMapper* get_vid_mapper(void* vid_mapper, void* query_config) {
@@ -408,8 +450,8 @@ void GatherVariantCalls::operate_on_columnar_cell(const GenomicsDBColumnarCell&
                                                    const VariantArraySchema& schema) {
   if (cell.at_new_query_column_interval()) {
     auto curr_query_column_interval_idx = cell.get_current_query_column_interval_idx();
-    auto begin = std::max<int64_t>(query_config.get_column_begin(curr_query_column_interval_idx), 0);
-    auto end = std::min<int64_t>(query_config.get_column_end(curr_query_column_interval_idx), INT64_MAX-1);
+    auto begin = query_config.get_num_column_intervals()>0?query_config.get_column_begin(curr_query_column_interval_idx):0;
+    auto end = query_config.get_num_column_intervals()>0?query_config.get_column_end(curr_query_column_interval_idx):INT64_MAX-1;
     // TODO: Change Column/RowRange to be uint64_t intervals instead of int64_t, so we don't have to typecast
     m_variant_call_processor.process(std::make_pair<uint64_t, uint64_t>((uint64_t)begin, (uint64_t)end));
   }
@@ -432,27 +474,32 @@ void GatherVariantCalls::operate_on_columnar_cell(const GenomicsDBColumnarCell&
                                       std::make_pair(contig_position, contig_position+end_position-coords[1]));
 
   std::vector<genomic_field_t> genomic_fields;
-  std::string ref_value;
-  std::string alt_value;
+  
   // Ignore first field as it is "END"
   for (auto i=1u; i<query_config.get_num_queried_attributes(); i++) {
     if (cell.is_valid(i)) {
-      genomic_field_t field_vec(query_config.get_query_attribute_name(i),
-                                cell.get_field_ptr_for_query_idx(i),
-                                cell.get_field_length(i));
-      genomic_fields.push_back(field_vec);
-
-      if(query_config.get_query_attribute_name(i) == "REF") {
-        ref_value = field_vec.str_value();
-      } else if(query_config.get_query_attribute_name(i) == "ALT" && field_vec.get_num_elements() >= 1) {
-        alt_value = field_vec.str_value().substr(0, field_vec.str_value().find_first_of("|"));
-      }
+      genomic_field_t field(query_config.get_query_attribute_name(i),
+                            cell.get_field_ptr_for_query_idx(i),
+                            cell.get_field_length(i));
+      genomic_fields.push_back(field);
     }
   }
 
-  if(!alt_value.empty() && m_annotation_service) {
-    AnnotationService* annotation_service = TO_ANNOTATION_SERVICE(m_annotation_service);
-    annotation_service->annotate(genomic_interval, ref_value, alt_value, genomic_fields);
+  if (m_annotation_service) {
+    std::string alt_value;
+    std::string ref_value;
+    for (auto field: genomic_fields) {
+      if (field.name == "REF") {
+        ref_value = field.str_value();
+      } else if (field.name == "ALT" && field.get_num_elements() >= 1) {
+        alt_value = field.str_value().substr(0, field.str_value().find_first_of("|"));
+      }
+    }
+
+    if (!alt_value.empty()) {
+      AnnotationService* annotation_service = TO_ANNOTATION_SERVICE(m_annotation_service);
+      annotation_service->annotate(genomic_interval, ref_value, alt_value, genomic_fields);
+    }
   }
 
   std::string sample_name;
@@ -531,7 +578,7 @@ void GenomicsDB::generate_vcf(const std::string& array,
   query_config.set_reference_genome(reference_genome);
   query_config.set_vcf_header_filename(vcf_header);
 
-  query_config.validate();
+  query_config.validate(m_concurrency_rank);
 
   generate_vcf(array, &query_config, output, output_format, overwrite);
 }
@@ -737,6 +784,60 @@ void GenomicsDBResults<genomicsdb_variant_call_t>::free() {
   // NOP: free'ing is done at the Variant(genomicsdb_variant_t) level
 }
 
+#define PIPED_SEP '|'
+#define SLASHED_SEP '/'
+std::string_view get_segment(std::string_view str, int segment_pos) {
+  std::string::size_type pos;
+  std::string::size_type next_pos = -1;
+  for (int j=0; j<segment_pos; j++) {
+    pos = next_pos + 1;
+    next_pos = str.find(PIPED_SEP);
+    if (next_pos == std::string::npos) {
+      if (j != segment_pos-1) {
+        return ".";
+      } else {
+        return str.substr(pos, str.length());
+      }
+    }
+  }
+  return str.substr(pos, next_pos);
+}
+
+std::string resolve(std::vector<int32_t>& gt, std::string_view ref, std::string_view alt) {
+  std::string resolved_gt;
+  for (auto i=0ul; i<gt.size(); i++) {
+    if (i%2) { // Phase
+      if (gt[i]) resolved_gt += PIPED_SEP; else resolved_gt += SLASHED_SEP;
+    } else if (gt[i] > 0) { // ALT
+      resolved_gt += get_segment(alt, gt[i]);
+    } else if (gt[i] == 0) { // REF
+      resolved_gt += ref;
+    } else { // UNKNOWN
+      resolved_gt += ".";
+    }
+  }
+  return resolved_gt;
+}
+
+const std::string resolve_gt(const std::vector<genomic_field_t>& genomic_fields)  {
+  std::string ref;
+  std::string alt;
+  std::vector<int32_t> gt_vec;
+  for (auto genomic_field: genomic_fields) {
+    if (genomic_field.name == "REF") {
+      ref = genomic_field.str_value();
+    } else if (genomic_field.name == "ALT") {
+      alt = genomic_field.str_value();
+    } else if (genomic_field.name == "GT") {
+      for (auto i=0u; i<genomic_field.num_elements; i++) {
+        gt_vec.push_back(genomic_field.int_value_at(i));
+      }
+    }
+  }
+  return resolve(gt_vec, ref, alt);
+}
+
+
 void GenomicsDBVariantCallProcessor::process(const std::string& sample_name,
                                              const int64_t* coords,
                                              const genomic_interval_t& genomic_interval,
