@@ -2,6 +2,7 @@
  * The MIT License (MIT)
  * Copyright (c) 2016-2017 Intel Corporation
  * Copyright (c) 2019-2020 Omics Data Automation, Inc.
+ * Copyright (c) 2023 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -439,8 +440,11 @@ bool BroadCombinedGVCFOperator::remap_if_needed_and_combine(const Variant& varia
       && check_if_too_many_alleles_and_print_message(variant, *(length_descriptors[0u])))
     return false;
   //Remapper for m_remapped_variant
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Warray-bounds"
   RemappedVariant remapper_variants[2] = { RemappedVariant(m_remapped_variant, query_field_idxs[0u], true),
     RemappedVariant(m_remapped_variant, query_field_idxs[1u], true) };
+#pragma GCC diagnostic pop
   //Get HistogramFieldHandlerBase ptr for histogram combine operation
   HistogramFieldHandlerBase* handler_ptr = 0;
   if(is_histogram) {
@@ -659,7 +663,12 @@ void BroadCombinedGVCFOperator::handle_FORMAT_fields(const Variant& variant) {
   //Pointer to extended vector inside field handler object
   const void* ptr = 0;
   uint64_t num_elements = 0ull;
-  int* int_vec = 0;     //will point to same address as ptr, but as int*
+  int* GT_vec = 0; // will point to same address as ptr, but as int*
+  int* DP_vec = 0; // ditto
+  //Hold values to figure out if any samples have GQ==0 and PL[0]==0 after the loop
+  //see https://github.com/broadinstitute/gatk/pull/8715
+  std::set<int> GQ_zero_values;
+  std::set<int> PL_zero_values;
   //Handle all fields - simply need to extend to the largest size
   for (auto i=0u; i<m_FORMAT_fields_vec.size(); ++i) {
     auto& curr_tuple = m_FORMAT_fields_vec[i];
@@ -689,20 +698,25 @@ void BroadCombinedGVCFOperator::handle_FORMAT_fields(const Variant& variant) {
       auto do_insert = !(m_query_config->sites_only_query());    //by default, insert into VCF record if !sites_only
       switch (known_field_enum) {
       case GVCF_GT_IDX: { //GT field is a pita
-        int_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
+        GT_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
         auto num_elements_per_sample = num_elements/variant.get_num_calls();
         //GT of type 0/2 is stored as [0,0,2] in GenomicsDB if phased ploidy is used, else [0, 2]
         //GT of type 0|2 is stored as [0,1,2] in GenomicsDB if phased ploidy is used, else [0, 2]
         auto max_ploidy = length_descriptor.get_ploidy(num_elements_per_sample);
         uint64_t output_idx = 0ull;
         for (j=0ull; j<num_elements; j+=num_elements_per_sample)
-          (*m_encode_GT_vector_function_ptr)(int_vec, j, num_elements_per_sample, output_idx);
+          (*m_encode_GT_vector_function_ptr)(GT_vec, j, num_elements_per_sample, output_idx);
         num_elements = max_ploidy*variant.get_num_calls();
         break;
       }
-      case GVCF_GQ_IDX:
+      case GVCF_GQ_IDX: {
+        int *GQ_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
+        for (auto i=0ul; i<variant.get_num_calls(); i++) {
+          if (*(GQ_vec+i) == 0) GQ_zero_values.insert(i);
+        }
         do_insert = m_should_add_GQ_field;
         break;
+      }
       case GVCF_MIN_DP_IDX: //simply copy over min-dp values
         memcpy_s(&(m_MIN_DP_vector[0]), m_remapped_variant.get_num_calls()*sizeof(int),
                  ptr, m_remapped_variant.get_num_calls()*sizeof(int));
@@ -715,10 +729,19 @@ void BroadCombinedGVCFOperator::handle_FORMAT_fields(const Variant& variant) {
         do_insert = false; //Do not insert DP_FORMAT, wait till DP is read
         break;
       case GVCF_DP_IDX:
-        int_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
+        DP_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
         valid_DP_found = true;
         do_insert = false; //Do not insert DP, handle DP related garbage at the end
         break;
+      case GVCF_PL_IDX: {
+        int *PL_vec = const_cast<int*>(reinterpret_cast<const int*>(ptr));
+        for (auto i=0ul; i<variant.get_num_calls(); i++) {
+          if (*(PL_vec+i*num_elements/variant.get_num_calls()) == 0) {
+            PL_zero_values.insert(i);
+          }
+        }
+        break;
+      }
       default:
         break;
       }
@@ -729,12 +752,36 @@ void BroadCombinedGVCFOperator::handle_FORMAT_fields(const Variant& variant) {
       }
     }
   }
+  // Set GT to . when GQ==0 and PL[0]==0 see https://github.com/broadinstitute/gatk/pull/8715
+  // TODO: The java tests for mixed ploid and with GQ and PL[0] as zero are ignoring ploidy for the
+  //       samples having haploid in the t0_haploid_triploid_1_2_3_triploid_deletion_java_produce_GT
+  //       tests. Does not seem to affect gatk, need to look at this.
+  if (m_query_config->produce_GT_field() && GQ_zero_values.size() > 0 && PL_zero_values.size() > 0) {
+    int32_t *GT_arr = NULL, num_GT_arr = 0;
+    for (auto i=0ul; i<variant.get_num_calls(); i++) {
+      if (GQ_zero_values.find(i) == GQ_zero_values.end() || PL_zero_values.find(i) == PL_zero_values.end()) continue;
+      if (!GT_arr && !num_GT_arr) {
+        auto ngt = bcf_get_genotypes(m_vcf_hdr, m_bcf_out, &GT_arr, &num_GT_arr);
+        if (ngt <= 0) GT_arr == NULL;
+      }
+      if (GT_arr) {
+        auto ploidy = num_GT_arr/variant.get_num_calls();
+        for (auto j= 0ul; j<ploidy; j++) {
+          if (GT_arr[i*ploidy+j] == bcf_int32_vector_end) break;
+          GT_arr[i*ploidy+j] = bcf_gt_missing;
+        }
+      }
+    }
+    if (GT_arr && num_GT_arr) {
+      bcf_update_genotypes(m_vcf_hdr, m_bcf_out, GT_arr, num_GT_arr);
+    }
+  }
   //Update DP fields
   if (valid_DP_found || valid_DP_FORMAT_found) {
     int64_t sum_INFO_DP = 0;
     auto found_one_valid_DP_FORMAT = false;
     for (auto j=0ull; j<m_remapped_variant.get_num_calls(); ++j) {
-      int dp_info_val = valid_DP_found ? int_vec[j] : get_bcf_missing_value<int>();
+      int dp_info_val = valid_DP_found ? DP_vec[j] : get_bcf_missing_value<int>();
       int dp_format_val = valid_DP_FORMAT_found ? m_DP_FORMAT_vector[j] : get_bcf_missing_value<int>();
       assert(is_bcf_valid_value<int>(dp_format_val) || dp_format_val == get_bcf_missing_value<int>());
       assert(is_bcf_valid_value<int>(dp_info_val) || dp_info_val == get_bcf_missing_value<int>());
