@@ -3,6 +3,7 @@
  *
  * The MIT License (MIT)
  * Copyright (c) 2019-2023 Omics Data Automation, Inc.
+ * Copyright (c) 2023 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -30,7 +31,6 @@
 #include "genomicsdb.h"
 #include "genomicsdb_config_base.h"
 #include "tiledb_utils.h"
-#include "genomicsdb_logger.h" // REMOVE
 
 #include "genomicsdb_export_config.pb.h"
 
@@ -63,11 +63,6 @@ TEST_CASE_METHOD(TempDir, "utils", "[genomicsdb_utils]") {
   CHECK(genomicsdb::read_entire_file(filename+".another", (void **)txt, &length) == TILEDB_ERR);
 }
 
-TEST_CASE("api get_version", "[get_version]") {
-  REQUIRE(genomicsdb_version().size() > 0);
-  REQUIRE_THAT(genomicsdb_version(), Catch::Equals(GENOMICSDB_VERSION));
-}
-
 TEST_CASE("api empty_args", "[empty_args]") {
   std::string empty_string;
 
@@ -106,6 +101,9 @@ static std::string loader_json(ctests_input_dir+"loader.json");
 static std::string workspace_PP(ctests_input_dir+"ws_phased_ploidy");
 static std::string vid_mapping_PP(ctests_input_dir+"vid_phased_ploidy.json");
 
+// Test workspace created with vcf2genomicsdb_init
+static std::string workspace_new(ctests_input_dir+"ws_new");
+
 static std::string array("t0_1_2");
 
 // Test Shared PosixFS Optimizations
@@ -286,6 +284,7 @@ class CountCellsProcessor : public GenomicsDBVariantCallProcessor {
   };
 
   void process(const interval_t& interval) {
+    m_intervals++;
   };
 
   void process(const std::string& sample_name,
@@ -293,11 +292,20 @@ class CountCellsProcessor : public GenomicsDBVariantCallProcessor {
                const genomic_interval_t& genomic_interval,
                const std::vector<genomic_field_t>& genomic_fields) {
     m_count++;
-    m_coordinates = coordinates;
+    m_coordinates[0] = coordinates[0];
+    m_coordinates[1] = coordinates[1];
   };
 
+  void check_single_match() {
+    CHECK(m_intervals == 1);
+    CHECK(m_count == 1);
+    CHECK(m_coordinates[0] == 1);
+    CHECK(m_coordinates[1] == 17384);
+  }
+
+  int m_intervals = 0;
   int m_count = 0;
-  const int64_t* m_coordinates;
+  int64_t m_coordinates[2];
 };
 
 class OneQueryIntervalProcessor : public GenomicsDBVariantCallProcessor {
@@ -313,10 +321,21 @@ class OneQueryIntervalProcessor : public GenomicsDBVariantCallProcessor {
     CHECK(m_sample_found);
   }
 
+  void reinitialize(int max_int_check =false) {
+    m_num_query_intervals = 0;
+    m_processed_rows = 0;
+    m_sample_found = false;
+    m_max_int_check = max_int_check;
+  }
+
   void process(const interval_t& interval) {
     m_num_query_intervals++;
     CHECK(interval.first == 0);
-    CHECK(interval.second == 1000000000);
+    if (m_max_int_check) {
+      CHECK(interval.second == INT64_MAX - 1);
+    } else {
+      CHECK(interval.second == 1000000000);
+    }
   };
 
   void process(const std::string& sample_name,
@@ -337,7 +356,7 @@ class OneQueryIntervalProcessor : public GenomicsDBVariantCallProcessor {
       if (m_attributes_size) {
         CHECK(genomic_fields.size() == m_attributes_size + 2); // Add '2' for REF and ALT fields
       }
-      int found_fields = 0;
+      size_t found_fields = 0;
       for (auto i=0u; i<genomic_fields.size(); i++) {
         if (genomic_fields[i].name == "REF") {
           found_fields++;
@@ -384,11 +403,12 @@ class OneQueryIntervalProcessor : public GenomicsDBVariantCallProcessor {
     }
   };
 
-  int m_attributes_size;
+  size_t m_attributes_size;
   int m_num_query_intervals = 0;
   int m_processed_rows = 0;
   int m_sample_found = false;
   int m_is_PP;
+  bool m_max_int_check = false;
 };
 
 class TwoQueryIntervalsProcessor : public GenomicsDBVariantCallProcessor {
@@ -400,6 +420,11 @@ class TwoQueryIntervalsProcessor : public GenomicsDBVariantCallProcessor {
     CHECK(m_processed_rows == 5);
   }
 
+  void reinitialize() {
+    m_num_query_intervals = 0;
+    m_processed_rows = 0;
+  }
+
   void process(const interval_t& interval) {
     m_num_query_intervals++;
     if (m_num_query_intervals == 1) {
@@ -437,11 +462,14 @@ TEST_CASE("api query_variant_calls direct", "[query_variant_calls_direct]") {
 
   OneQueryIntervalProcessor one_query_interval_processor(attributes);
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}}, {{0,3}});
+  one_query_interval_processor.reinitialize();
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}});
+  one_query_interval_processor.reinitialize(true);
   gdb->query_variant_calls(one_query_interval_processor, array);
 
   TwoQueryIntervalsProcessor two_query_intervals_processor;
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}}, {{0,3}});
+  two_query_intervals_processor.reinitialize();
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}});
 
   delete gdb;
@@ -460,11 +488,14 @@ TEST_CASE("api query_variant_calls direct DP and GT", "[query_variant_calls_dire
 
   OneQueryIntervalProcessor one_query_interval_processor(attributes);
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}}, {{0,3}});
+  one_query_interval_processor.reinitialize();
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}});
+  one_query_interval_processor.reinitialize(true);
   gdb->query_variant_calls(one_query_interval_processor, array);
 
   TwoQueryIntervalsProcessor two_query_intervals_processor;
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}}, {{0,3}});
+  two_query_intervals_processor.reinitialize();
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}});
 
   delete gdb;
@@ -483,11 +514,14 @@ TEST_CASE("api query_variant_calls direct DP and GT with PP", "[query_variant_ca
 
   OneQueryIntervalProcessor one_query_interval_processor(attributes, true);
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}}, {{0,3}});
+  one_query_interval_processor.reinitialize();
   gdb->query_variant_calls(one_query_interval_processor, array, {{0,1000000000}});
+  one_query_interval_processor.reinitialize(true);
   gdb->query_variant_calls(one_query_interval_processor, array);
 
   TwoQueryIntervalsProcessor two_query_intervals_processor;
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}}, {{0,3}});
+  two_query_intervals_processor.reinitialize();
   gdb->query_variant_calls(two_query_intervals_processor, array, {{0,17000},{17000,18000}});
 
   delete gdb;
@@ -498,12 +532,12 @@ TEST_CASE("api query_variant_calls with json", "[query_variant_calls_with_json]"
   gdb->query_variant_calls();
 
   OneQueryIntervalProcessor one_query_interval_processor;
-  gdb->query_variant_calls(one_query_interval_processor);
+  gdb->query_variant_calls(one_query_interval_processor, "", GenomicsDB::NONE);
   delete gdb;
 
   OneQueryIntervalProcessor another_query_interval_processor;
   gdb = new GenomicsDB(query_json, GenomicsDB::JSON_FILE, loader_json, 0);
-  gdb->query_variant_calls(another_query_interval_processor);
+  gdb->query_variant_calls(another_query_interval_processor, "", GenomicsDB::NONE);
   delete gdb;
 
   CHECK_THROWS_AS(new GenomicsDB(query_json, GenomicsDB::JSON_FILE, loader_json, 1), GenomicsDBConfigException);
@@ -548,17 +582,24 @@ TEST_CASE("api query_variant_calls with protobuf", "[query_variant_calls_with_pr
   std::string config_string;
   CHECK(config->SerializeToString(&config_string));
 
-  // no array name set, should throw exception
-  CHECK_THROWS_AS(new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0), GenomicsDBConfigException);
-
-  config->set_array_name(array);
-  CHECK(config->SerializeToString(&config_string));
-
+  OneQueryIntervalProcessor one_query_interval_processor;
+  
+  // Try with no array name set, GenomicsDB should find the single array t0_1_2 during query
   GenomicsDB* gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
   gdb->query_variant_calls();
-  OneQueryIntervalProcessor one_query_interval_processor;
-  gdb->query_variant_calls(one_query_interval_processor);
+  gdb->query_variant_calls(one_query_interval_processor, "", GenomicsDB::NONE);
   delete gdb;
+  
+  config->set_array_name(array);
+  CHECK(config->SerializeToString(&config_string));
+  gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+  gdb->query_variant_calls();
+  one_query_interval_processor.reinitialize();
+  gdb->query_variant_calls(one_query_interval_processor, "", GenomicsDB::NONE);
+  delete gdb;
+
+  // Filter expressions are supported only for workspaces/arrays with ploidy information
+  config->set_workspace(workspace_PP);
 
   SECTION("Try query with contig intervals instead of tiledb column intervals") {
     ContigInterval* contig_interval = new ContigInterval();
@@ -569,34 +610,486 @@ TEST_CASE("api query_variant_calls with protobuf", "[query_variant_calls_with_pr
     column_interval->set_allocated_contig_interval(contig_interval);
     CHECK(config->SerializeToString(&config_string));
     gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
-    gdb->query_variant_calls();
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 5);
     delete gdb;
   }
 
+  config->set_query_filter("REF == \"G\" && resolve(GT, REF, ALT) &= \"T/T\" && ALT |= \"T\"");
   SECTION("Try query with a filter") {
-    config->set_query_filter("REF == \"G\" && GT &= \"1/1\" && ALT |= \"T\"");
     CHECK(config->SerializeToString(&config_string));
     gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
     CountCellsProcessor count_cells_processor;
-    gdb->query_variant_calls(count_cells_processor);
-    CHECK(count_cells_processor.m_count == 1);
-    CHECK(count_cells_processor.m_coordinates[0] == 1);
-    CHECK(count_cells_processor.m_coordinates[1] == 17384);
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    count_cells_processor.check_single_match();
+    delete gdb;
+  }
+
+  config->set_workspace(workspace_PP);
+  SECTION("Try query with a filter and workspace with GT ploidy") {
+    config->set_bypass_intersecting_intervals_phase(true);
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    count_cells_processor.check_single_match();
     delete gdb;
   }
 
   SECTION("Try query with a filter and a small segment size to force TileDB buffers overflow") {
-    config->set_query_filter("REF == \"G\" && GT &= \"11\" && ALT |= \"T\"");
     config->set_segment_size(40);
     CHECK(config->SerializeToString(&config_string));
     gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
     CountCellsProcessor count_cells_processor;
-    gdb->query_variant_calls(count_cells_processor);
-    CHECK(count_cells_processor.m_count == 1);
-    CHECK(count_cells_processor.m_coordinates[0] == 1);
-    CHECK(count_cells_processor.m_coordinates[1] == 17384);
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    count_cells_processor.check_single_match();
     delete gdb;
   }
+
+  config->set_query_filter("POS==99999999 && ROW==1");
+  SECTION("Try query with POS and ROW with no match") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 0);
+    CHECK(count_cells_processor.m_count == 0);
+    delete gdb;
+  }
+
+
+  config->set_query_filter("POS==17384 && ROW==1 && ISHOMALT && resolve(GT, REF, ALT) &= \"T/T\"");
+  SECTION("Try query with POS and ROW") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    count_cells_processor.check_single_match();
+    delete gdb;
+  }
+  
+  config->set_query_filter("ISHOMREF");
+  SECTION("Try query with ISHOMREF") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 2);
+    delete gdb;
+  }
+
+  config->set_query_filter("!ISHOMREF");
+  SECTION("Try query with ISHOMREF") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 3);
+    delete gdb;
+  }
+
+  config->set_query_filter("ISHOMALT");
+  SECTION("Try query with ISHOMREF") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 1);
+    delete gdb;
+  }
+
+  config->set_query_filter("ISHET");
+  SECTION("Try query with ISHOMREF") {
+    CHECK(config->SerializeToString(&config_string));
+    gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 2);
+    delete gdb;
+  }
+
+}
+
+TEST_CASE("api query_variant_calls with protobuf and config", "[query_variant_calls_with_protobuf_and_explicit_configuration]") {
+  using namespace genomicsdb_pb;
+
+  ExportConfiguration *config = new ExportConfiguration();
+
+  config->set_workspace(workspace_new);
+  config->set_array_name("t0_1_2");
+  config->set_callset_mapping_file(workspace_new+"/callset.json");
+  config->set_vid_mapping_file(workspace_new+"/vidmap.json");
+  config->set_enable_shared_posixfs_optimizations(true);
+  config->set_bypass_intersecting_intervals_phase(true);
+  // query_attributes
+  config->add_attributes()->assign("REF");
+  config->add_attributes()->assign("ALT");
+  config->add_attributes()->assign("GT");
+  config->set_segment_size(128);
+
+  std::string config_string;
+  CHECK(config->SerializeToString(&config_string));
+  GenomicsDB *gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+
+  SECTION("Base configuration") {
+    CountCellsProcessor count_cells_processor;
+    gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 5);
+  }
+
+  SECTION("Subset Array with pb string that is not Query or ExportConfiguration") {
+    CountCellsProcessor count_cells_processor;
+    CHECK_THROWS_AS(gdb->query_variant_calls(count_cells_processor, "NotAProtobufConfiguration",
+                                             GenomicsDB::PROTOBUF_BINARY_STRING), std::exception);
+  }
+
+  SECTION("Subset Array using QueryConfiguration") {
+    CountCellsProcessor count_cells_processor;
+    QueryConfiguration query_config;
+    query_config.set_array_name("t0_1_2");
+    ContigInterval* contig_interval = query_config.add_query_contig_intervals();
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(1);
+    contig_interval->set_end(100000);
+    RowRangeList* row_ranges = query_config.add_query_row_ranges();
+    RowRange* row_range = row_ranges->add_range_list();
+    row_range->set_low(0);
+    row_range->set_high(3);
+    std::string query_string;
+    CHECK(query_config.SerializeToString(&query_string));
+    gdb->query_variant_calls(count_cells_processor, query_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 5);
+  }
+
+  SECTION("Subset Array using ExportConfiguration") {
+    CountCellsProcessor count_cells_processor;
+    ExportConfiguration query_config;
+    ContigInterval* contig_interval = query_config.add_query_contig_intervals();
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(1);
+    contig_interval->set_end(100000);
+    std::string query_string;
+    CHECK(query_config.SerializeToString(&query_string));
+    gdb->query_variant_calls(count_cells_processor, query_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 5);
+  }
+
+  SECTION("Subset Array using QueryConfiguration - two intervals") {
+    CountCellsProcessor count_cells_processor;
+    QueryConfiguration query_config;
+    ContigInterval* contig_interval = query_config.add_query_contig_intervals();
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(1);
+    contig_interval->set_end(17000);
+    contig_interval = query_config.add_query_contig_intervals();
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(17000);
+    contig_interval->set_end(18000);
+    std::string query_string;
+    CHECK(query_config.SerializeToString(&query_string));
+    gdb->query_variant_calls(count_cells_processor, query_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+    CHECK(count_cells_processor.m_intervals == 2);
+    CHECK(count_cells_processor.m_count == 5);
+  }
+
+   SECTION("Subset Array using QueryConfiguration - multiple queries") {
+    QueryConfiguration query_config;
+
+    // First query
+    CountCellsProcessor count_cells_processor;
+    ContigInterval* contig_interval = query_config.add_query_contig_intervals();
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(1);
+    contig_interval->set_end(17000);
+    std::string query_string;
+    CHECK(query_config.SerializeToString(&query_string));
+    gdb->query_variant_calls(count_cells_processor, query_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+    CHECK(count_cells_processor.m_intervals == 1);
+    CHECK(count_cells_processor.m_count == 2);
+
+    // Second query
+    CountCellsProcessor another_count_cells_processor;
+    contig_interval->set_contig("1");
+    contig_interval->set_begin(17000);
+    contig_interval->set_end(18000);
+    CHECK(query_config.SerializeToString(&query_string));
+    gdb->query_variant_calls(another_count_cells_processor, query_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+    CHECK(another_count_cells_processor.m_intervals == 1);
+    CHECK(another_count_cells_processor.m_count == 3);
+  }
+
+  delete gdb;
+}
+
+TEST_CASE("api query_variant_calls with protobuf new", "[query_variant_calls_with_protobuf_new]") {
+  using namespace genomicsdb_pb;
+
+  ExportConfiguration *config = new ExportConfiguration();
+
+  config->set_workspace(workspace_new);
+  config->set_array_name("t0_1_2");
+  config->set_callset_mapping_file(workspace_new+"/callset.json");
+  config->set_vid_mapping_file(workspace_new+"/vidmap.json");
+
+  // query_row_ranges
+  RowRangeList* row_ranges = config->add_query_row_ranges();
+  RowRange* row_range = row_ranges->add_range_list();
+  row_range->set_low(0);
+  row_range->set_high(3);
+
+  // query contig_interval
+  ContigInterval* contig_interval = config->add_query_contig_intervals();
+  contig_interval->set_contig("1");
+  contig_interval->set_begin(1);
+  contig_interval->set_end(249250621);
+
+  // query_attributes
+  config->add_attributes()->assign("GT");
+  config->add_attributes()->assign("DP");
+
+  config->set_bypass_intersecting_intervals_phase(true);
+  config->set_query_filter("REF == \"G\" && resolve(GT, REF, ALT) &= \"T/T\" && ALT |= \"T\"");
+
+  size_t segment_sizes[4] = { 16, 36, 96, 128 };
+
+  for (auto i=0; i<4; i++) {
+    SECTION("Test filter test for " + std::to_string(i)) {
+      config->set_segment_size(segment_sizes[i]);
+
+      std::string config_string;
+      CHECK(config->SerializeToString(&config_string));
+      GenomicsDB *gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+
+      CountCellsProcessor count_cells_processor;
+      gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+
+      CHECK(count_cells_processor.m_intervals == 1);
+      CHECK(count_cells_processor.m_count == 1);
+      delete gdb;
+    }
+  }
+}
+
+TEST_CASE("api query_variant_calls with JSONVariantCallProcessor", "[query_variant_calls_with_json_processor]") {
+  using namespace genomicsdb_pb;
+
+  ExportConfiguration *config = new ExportConfiguration();
+
+  config->set_workspace(workspace_new);
+  config->set_array_name("t0_1_2");
+  config->set_callset_mapping_file(workspace_new+"/callset.json");
+  config->set_vid_mapping_file(workspace_new+"/vidmap.json");
+
+  // query_row_ranges
+  RowRangeList* row_ranges = config->add_query_row_ranges();
+  RowRange* row_range = row_ranges->add_range_list();
+  row_range->set_low(0);
+  row_range->set_high(3);
+
+  // query contig_interval
+  ContigInterval* contig_interval = config->add_query_contig_intervals();
+  contig_interval->set_contig("1");
+  contig_interval->set_begin(1);
+  contig_interval->set_end(249250621);
+
+  // query_attributes
+  config->add_attributes()->assign("GT");
+  config->add_attributes()->assign("DP");
+
+  config->set_bypass_intersecting_intervals_phase(true);
+
+  config->set_segment_size(128);
+
+  std::string config_string;
+  CHECK(config->SerializeToString(&config_string));
+  GenomicsDB *gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+
+  JSONVariantCallProcessor json_processor;
+  gdb->query_variant_calls(json_processor, "", GenomicsDB::NONE);
+  auto output = json_processor.construct_json_output();
+  // {"HG00141":{"CHR":["1","1"],"POS":[12141,17385],"DP":[null,null],"GT":["C/C","G/A"],"REF":["C","G"]},
+  //  "HG01530":{"CHR":["1"],"POS":[17385],"DP":[76],"GT":["G/A"],"REF":["G"]},
+  //  "HG01958":{"CHR":["1","1"],"POS":[12145,17385],"DP":[null,120],"GT":["C/C","T/T"],"REF":["C","G"]}}
+  CHECK(output.length() == 273);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  JSONVariantCallProcessor json_processor0(JSONVariantCallProcessor::all_by_calls);
+  gdb->query_variant_calls(json_processor0, "", GenomicsDB::NONE);
+  output = json_processor0.construct_json_output();
+  // {"FIELD":["CHR","POS","DP","GT","REF"],
+  //  "HG00141":[["1",12141,null,"C/C","C"],["1",17385,null,"G/A","G"]],
+  //  "HG01530":[["1",17385,76,"G/A","G"]],
+  //  "HG01958":[["1",12145,null,"C/C","C"],["1",17385,120,"T/T","G"]]}
+  CHECK(output.length() == 207);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  JSONVariantCallProcessor json_processor1(JSONVariantCallProcessor::samples_with_ncalls);
+  gdb->query_variant_calls(json_processor1, "", GenomicsDB::NONE);
+  output = json_processor1.construct_json_output();
+  // {"HG00141":2,"HG01530":1,"HG01958":2}
+  CHECK(output.length() == 37);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  JSONVariantCallProcessor json_processor2(JSONVariantCallProcessor::just_ncalls);
+  gdb->query_variant_calls(json_processor2, "", GenomicsDB::NONE);
+  output = json_processor2.construct_json_output();
+  // {"num_calls":5}
+  CHECK(output.length()== 15);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  JSONVariantCallProcessor json_processor3;
+  json_processor3.set_payload_mode(JSONVariantCallProcessor::just_ncalls);
+  gdb->query_variant_calls(json_processor3, "", GenomicsDB::NONE);
+  output = json_processor3.construct_json_output();
+  // {"num_calls":5}
+  CHECK(output.length() == 15);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  JSONVariantCallProcessor json_processor4;
+  json_processor4.set_payload_mode(JSONVariantCallProcessor::just_samples);
+  gdb->query_variant_calls(json_processor4, "", GenomicsDB::NONE);
+  output = json_processor4.construct_json_output();
+  // ["HG00141","HG01530","HG01958"]
+  CHECK(output.length() == 31);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+
+  delete gdb;
+
+  config->clear_attributes();
+  CHECK(config->SerializeToString(&config_string));
+  gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+  JSONVariantCallProcessor json_processor5(JSONVariantCallProcessor::all);
+  gdb->query_variant_calls(json_processor5, "", GenomicsDB::NONE);
+  output = json_processor5.construct_json_output();
+  // {"HG00141":{"CHR":["1","1"],"POS":[12141,17385],"AD":[null,58],"BaseQRankSum":[null,-2.0959999561309816],
+  //             "ClippingRankSum":[null,-1.8589999675750733],"DP":[null,null],"DP_FORMAT":[2,80],"DS":["1","1"],
+  //             "FILTER":[null,0],"GQ":[0,99],"GT":["C/C","G/A"],"HaplotypeScore":[null,null],"InbreedingCoeff":[null,null],
+  //             "MIN_DP":[0,null],"MLEAC":[null,1],"MLEAF":[null,0.5],"MQ":[null,31.719999313354493],"MQ0":[null,8],
+  //             "MQRankSum":[null,-0.32899999618530276],"PGT":[null,"0|1"],"PID":[null,"17385_G_A"],"PL":[0,504],
+  //             "QUAL":[null,475.7699890136719],"RAW_MQ":[null,5.5],"REF":["C","G"],
+  //             "ReadPosRankSum":[null,0.004999999888241291],"SB":[null,"[58, 0, 22, 0]"]},
+  //  "HG01530":{"CHR":["1"],"POS":[17385],"AD":[40],"BaseQRankSum":[1.0460000038146973],"ClippingRankSum":[-2.242000102996826],
+  //             "DP":[76],"DP_FORMAT":[76],"DS":[null],"FILTER":[null],"GQ":[99],"GT":["G/A"],"HaplotypeScore":[null],
+  //             "InbreedingCoeff":[null],"MIN_DP":[null],"MLEAC":[1],"MLEAF":[0.5],"MQ":[59.369998931884769],"MQ0":[0],
+  //             "MQRankSum":[-0.4320000112056732],"PGT":[null],"PID":[null],"PL":[1018],"QUAL":[989.77001953125],
+  //             "RAW_MQ":[null],"REF":["G"],"ReadPosRankSum":[2.055000066757202],"SB":["[9, 31, 13, 23]"]},
+  //  "HG01958":{"CHR":["1","1"],"POS":[12145,17385],"AD":[null,0],"BaseQRankSum":[null,-2.0739998817443849],
+  //             "ClippingRankSum":[null,0.5550000071525574],"DP":[null,120],"DP_FORMAT":[3,120],"DS":["1",null],
+  //             "FILTER":[null,0],"GQ":[0,99],"GT":["C/C","T/T"],"HaplotypeScore":[null,null],"InbreedingCoeff":[null,null],
+  //             "MIN_DP":[0,null],"MLEAC":[null,2],"MLEAF":[null,1.0],"MQ":[null,29.81999969482422],"MQ0":[null,3],
+  //             "MQRankSum":[null,-1.36899995803833],"PGT":[null,"0|1"],"PID":[null,"17385_G_T"],"PL":[0,3336],
+  //             "QUAL":[null,3302.77001953125],"RAW_MQ":[null,2.5],"REF":["C","G"],"ReadPosRankSum":[null,-0.10100000351667404],
+  //             "SB":[null,"[0, 0, 0, 0]"]}}
+  CHECK(output.length() == 1805);
+  printf("%lu %s\n\n", output.length(), output.c_str());
+  delete gdb;
+
+  // Test return of empty output
+  config->clear_query_contig_intervals();
+  contig_interval = config->add_query_contig_intervals();
+  contig_interval->set_contig("22");
+  
+  CHECK(config->SerializeToString(&config_string));
+  gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+  for (auto payload=0; payload < 5; payload++) {
+    JSONVariantCallProcessor json_processor;
+    json_processor.set_payload_mode(static_cast<JSONVariantCallProcessor::payload_t>(payload));
+    gdb->query_variant_calls(json_processor, "", GenomicsDB::NONE);
+    output = json_processor.construct_json_output();
+    if (payload == JSONVariantCallProcessor::just_ncalls) {
+      CHECK(output == "{\"num_calls\":0}");
+    } else if (payload == JSONVariantCallProcessor::just_samples) {
+      CHECK(output == "[]");
+    } else {
+      CHECK(output == "{}");
+    }
+  }
+  delete gdb;
+}
+
+
+TEST_CASE("Test genomicsdb demo test case", "[genomicsdb_demo]") {
+  using namespace genomicsdb_pb;
+  
+  char *genomicsdb_demo_workspace = getenv("GENOMICSDB_DEMO_WS");
+  if (!genomicsdb_demo_workspace) return;
+
+  ExportConfiguration *config = new ExportConfiguration();
+
+  std::string ws(genomicsdb_demo_workspace);
+  config->set_workspace(ws);
+  config->set_array_name("allcontigs$1$3095677412");
+  config->set_callset_mapping_file(ws+"/callset.json");
+  config->set_vid_mapping_file(ws+"/vidmap.json");
+
+  // query_contig_intervals
+  auto* contig_interval = config->add_query_contig_intervals();
+  contig_interval->set_contig("17");
+  contig_interval->set_begin(7571719);
+  contig_interval->set_end(7590868);
+
+  // query_row_ranges
+  auto* row_range = config->add_query_row_ranges()->add_range_list();
+  row_range->set_low(0);
+  row_range->set_high(200000);
+
+  // query_attributes
+  config->add_attributes()->assign("REF");
+  config->add_attributes()->assign("ALT");
+  config->add_attributes()->assign("GT");
+
+  // other
+  config->set_bypass_intersecting_intervals_phase(true);
+  config->set_enable_shared_posixfs_optimizations(true);
+
+  // filters
+  std::vector<std::string> filters = {"", // zlib arm64 - 1s
+    "REF==\"A\"", // 2s
+    "REF==\"A\" && ALT|=\"T\"", // 2s
+    "REF==\"A\" &&  ALT|=\"T\" && ISHOMALT", // 3s
+    "REF==\"A\" && ALT|=\"T\" && resolve(GT, REF, ALT) &= \"T|T\"" // 3s
+  };
+
+  // sizes
+  std::vector<size_t> segment_sizes = { 0, 10240, 20480, 40960 };
+
+  // results
+  std::vector<int64_t> counts = { 2962039, 400432, 82245, 82245, 69548 };
+
+  for (auto i=0u; i<filters.size(); i++) {
+    for (auto j=0u; j<segment_sizes.size(); j++) {
+      SECTION("Demo test for " + std::to_string(i) + std::to_string(j)) {
+        if (segment_sizes[j] > 0) config->set_segment_size(segment_sizes[j]);
+        config->set_query_filter(filters[i]);
+
+        std::string config_string;
+        CHECK(config->SerializeToString(&config_string));
+
+        Catch::Timer t;
+        t.start();
+
+        GenomicsDB* gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+        CountCellsProcessor count_cells_processor;
+        gdb->query_variant_calls(count_cells_processor, "", GenomicsDB::NONE);
+
+        CHECK(count_cells_processor.m_intervals == 1);
+        CHECK(count_cells_processor.m_count == counts[i]);
+        printf("Elapsed Time=%us for filter=%s segment_size=%zu\n", t.getElapsedMilliseconds()/1000,
+               filters[i].c_str(), segment_sizes[j]);
+  
+        delete gdb;
+      }
+    }
+  }
 }
 
 TEST_CASE("api generate_vcf direct", "[query_generate_vcf_direct]") {
@@ -647,16 +1140,16 @@ TEST_CASE("api generate_vcf with json multiple threads", "[query_generate_with_j
     delete gdb;
   };
 
-  int num_threads = 8;
+  size_t num_threads = 8;
   std::vector<std::thread> threads;
-  for (auto i=0; i<num_threads; i++) {
+  for (auto i=0ul; i<num_threads; i++) {
     std::thread thread_object(test_genomicsdb_fn, i);
     threads.push_back(std::move(thread_object));
   }
 
   CHECK(num_threads == threads.size());
 
-  for (auto i=0; i<num_threads; i++) {
+  for (auto i=0ul; i<num_threads; i++) {
     threads[i].join();
   }
 }
@@ -691,7 +1184,7 @@ class VariantAnnotationCallProcessor : public GenomicsDBVariantCallProcessor {
            CHECK(genomic_fields[i].str_value() == "A|&");
          } else if(genomic_fields[i].name == "GT") {
            ++countExpectedGenomicFields;
-           CHECK(genomic_fields[i].str_value() == "");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type("GT")) == "0/1");
          } else if(genomic_fields[i].name == "dataSourceZero_field0") {
            ++countExpectedGenomicFields;
            CHECK(get_genomic_field_type(genomic_fields[i].name).is_int());
@@ -707,7 +1200,7 @@ class VariantAnnotationCallProcessor : public GenomicsDBVariantCallProcessor {
          } else if(genomic_fields[i].name == "dataSourceZero_field3") {
            ++countExpectedGenomicFields;
            CHECK(get_genomic_field_type(genomic_fields[i].name).is_string());
-           CHECK(genomic_fields[i].str_value() == "noot");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "noot");
          } else if(genomic_fields[i].name == "dataSourceZero_field7") {
            ++countExpectedGenomicFields;
            CHECK(get_genomic_field_type(genomic_fields[i].name).is_string());
@@ -715,7 +1208,7 @@ class VariantAnnotationCallProcessor : public GenomicsDBVariantCallProcessor {
          } else if(genomic_fields[i].name == "dataSourceZero_ID") {
            ++countExpectedGenomicFields;
            CHECK(get_genomic_field_type(genomic_fields[i].name).is_string());
-           CHECK(genomic_fields[i].str_value() == "id001");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "id001");
          } else {
            countExpectedGenomicFields = -1;
          }
@@ -738,13 +1231,13 @@ class VariantAnnotationCallProcessor : public GenomicsDBVariantCallProcessor {
            CHECK(genomic_fields[i].str_value() == "T|&");
          } else if(genomic_fields[i].name == "GT") {
            ++countExpectedGenomicFields;
-           CHECK(genomic_fields[i].str_value().length() == 1);
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type("GT")) == "1/1");
          } else if(genomic_fields[i].name == "DP") {
            ++countExpectedGenomicFields;
-           CHECK(genomic_fields[i].str_value() == "x");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "120");
          } else if(genomic_fields[i].name == "dataSourceZero_field3") {
            ++countExpectedGenomicFields;
-           CHECK(genomic_fields[i].str_value() == "waldo");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "waldo");
          } else if(genomic_fields[i].name == "dataSourceZero_field4") {
            ++countExpectedGenomicFields;
            CHECK(get_genomic_field_type(genomic_fields[i].name).is_int());
@@ -759,7 +1252,7 @@ class VariantAnnotationCallProcessor : public GenomicsDBVariantCallProcessor {
            CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "biz");
          } else if(genomic_fields[i].name == "dataSourceZero_ID") {
            ++countExpectedGenomicFields;
-           CHECK(genomic_fields[i].str_value() == "id002");
+           CHECK(genomic_fields[i].to_string(get_genomic_field_type(genomic_fields[i].name)) == "id002");
          } else {
            countExpectedGenomicFields = -1;
          }
@@ -853,14 +1346,14 @@ TEST_CASE("api annotate query_variant_calls with test datasource 0", "[annotate_
   GenomicsDB* gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
 
   VariantAnnotationCallProcessor variant_annotation_processor;
-  gdb->query_variant_calls(variant_annotation_processor);
+  gdb->query_variant_calls(variant_annotation_processor, "", GenomicsDB::NONE);
   delete gdb;
 
   // Check if exception thrown when annotation buffer size is too small to hold the annotated field values
   config->set_annotation_buffer_size(4);
   CHECK(config->SerializeToString(&config_string));
   gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
-  CHECK_THROWS_AS(gdb->query_variant_calls(variant_annotation_processor), GenomicsDBException);
+  CHECK_THROWS_AS(gdb->query_variant_calls(variant_annotation_processor, "", GenomicsDB::NONE), GenomicsDBException);
   delete gdb;
 
   // Add an info field that doesn't exist to the configuration; expect custom exception
@@ -868,6 +1361,6 @@ TEST_CASE("api annotate query_variant_calls with test datasource 0", "[annotate_
   annotation_source0->add_attributes()->assign("no_exist_field0");
   CHECK(config->SerializeToString(&config_string));
   gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING, loader_json, 0);
-  CHECK_THROWS_AS(gdb->query_variant_calls(variant_annotation_processor), GenomicsDBException);
+  CHECK_THROWS_AS(gdb->query_variant_calls(variant_annotation_processor, "", GenomicsDB::NONE), GenomicsDBException);
   delete gdb;
 }
